/********************************************************
 * 2D Data Visualization - Cuisines by Country          *
 * Tested in Processing 3                               *     
 *                                                      *
 * Author: Chantal Nguyen                               *
 * Supervisor: George Legrady                           *
 *                                                      *
 * Purpose: Visualize the most popular cuisines as      *
 *          determined by the number of SPL checkouts.  *
 *                                                      *
 * Usage:                                               *
 *          Press any key to toggle between evenly-     *
 *          spaced and proportionally-spaced layouts.   *
 *          Rollover circle to plot corresponding       *
 *          number of checkouts per month and show      *
 *          top 8 items for the corresponding cuisine.  *
 *                                                      *
 ********************************************************/

import java.util.Collections;

PFont f1;
PFont f2;
PFont f3;

Table country_data;
Table top_items;
int rows, cols;

float[][] data_matrix;

float max, min, max_counts, max_item_count;

float[] count_sums; 

float r;
float max_rad = 550;
int vertical_offset = 75;

ArrayList<country> c1 = new ArrayList<country>();

float Xdif;
float Ydif;

boolean toggle = true;

color cc1 = #D8B6B4;
color cc2 = #4C8493;
color cc3 = #FAC856;
color cc4 = #6E987B;
color cc5 = #BB4F24;
color cc6 = #3D89A7;
color cc7 = #CDA91D;
color cc8 = #841F11;
color cc9 = #C7904E;
color cc10 = #335144;
color cc11 = #C38C66;
color cc12 = #D31719;
color cc13 = #3C3B54;
color cc14 = #D28F33;
color cc15 = #133D30;

color[] colors = {cc1, cc2, cc3, cc4, cc5, cc6, cc7, cc8, cc9, cc10, cc11, cc12, cc13, cc14, cc15};

int lim = 20; // how many circles to plot


void setup() {
  fullScreen();

  f1 = createFont("Futura-Bold", 48);
  f2 = createFont("Futura-Bold", 16);
  f3 = createFont("Futura-Bold", 64);

  country_data = loadTable("countries.csv"); // keep header as it includes country names

  rows = country_data.getRowCount()-1;
  cols = country_data.getColumnCount();

  data_matrix = new float[cols-2][rows]; // initialize array

  // import table data into matrix
  // transpose matrix so that each row is a country and each column is a month
  for (int i=0; i<data_matrix.length; i++) {
    for (int j=0; j<data_matrix[0].length; j++) {
      data_matrix[i][j] = country_data.getFloat(j+1, i+2);
    }
  }

  // sum counts over months for each country
  count_sums = new float[data_matrix.length];
  for (int i=0; i<data_matrix.length; i++) {
    for (int j=0; j<data_matrix[0].length; j++) {
      count_sums[i] += data_matrix[i][j];
    }
    if (count_sums[i] > max_counts) {
      max_counts = count_sums[i];
    }
  }

  // get top 8 books for each country
  top_items = loadTable("items.csv", "header");

  // determine which individual item has the most counts
  for (int i=0; i<top_items.getRowCount(); i++) {
    if (top_items.getFloat(i, 2) > max_item_count) {
      max_item_count = top_items.getFloat(i, 2);
    }
  }

  // populate country list
  for (int i=0; i<data_matrix.length; i++) {
    ArrayList<lib_item> tempList = new ArrayList<lib_item>();
    for (int j=0; j<8; j++) {
      tempList.add(new lib_item(top_items.getString(i*8+j, 0), top_items.getString(i*8+j, 1), top_items.getFloat(i*8+j, 2)));
    }
    c1.add(new country(i, country_data.getString(0, i+2), count_sums[i], tempList));
  }
 
  // sort country list by descending number of counts
  // id is preserved as original row number in data_matrix
  Collections.sort(c1, Collections.reverseOrder());
  
}


class country implements Comparable<country> {
  int id;
  String country_name;
  float checkout_counts;
  ArrayList<lib_item> items;
  country(int id_, String country_name_, float checkout_counts_, ArrayList items_) {
    id = id_;
    country_name = country_name_;
    checkout_counts = checkout_counts_;
    items = items_;
  }
  int compareTo(country c1)
  {
    country other=(country)c1;
    if (other.checkout_counts>checkout_counts)
      return -1;
    if (other.checkout_counts==checkout_counts)
      return 0;
    else
      return 1;
  }
}

class lib_item {
  String item_type;
  String title;
  float bibcounts;
  lib_item(String item_type_, String title_, float bibcounts_) {
    item_type = item_type_;
    title = title_;
    bibcounts = bibcounts_;
  }
}

void draw() {
  background(252);
  textFont(f2);
  fill(165);
  textAlign(LEFT, TOP);
  text("Press any key to toggle", 15, 15);
  if (toggle) {
    evenCircleLayout();
    rolloverPlot(toggle);
  } else {
    sqrtCircleLayout();
    rolloverPlot(toggle);
  }

  // draw year lines
  pushMatrix();
  translate(width/2, height/2-vertical_offset);
  for (int yr=2006; yr<=2018; yr++) {
    stroke(80);
    strokeWeight(0.5);
    line(0, 0, 300*cos(map(yr, 2006, 2018, -4*PI/3, PI/3)), 300*sin(map(yr, 2006, 2018, -4*PI/3, PI/3)));
    textFont(f2);
    fill(0, 0.5*255);
    textAlign(CENTER);
    text(yr, 320*cos(map(yr, 2006, 2018, -4*PI/3, PI/3)), 320*sin(map(yr, 2006, 2018, -4*PI/3, PI/3)));
  }
  popMatrix();
}

boolean rollover(int mx, int my, boolean toggle) {
  float radius_buffer;
  if (toggle) { 
    radius_buffer = lim/7;
  } else {
    radius_buffer = 2;
  }
  if ((sq(Xdif)) + sq((Ydif)) <= sq(r+radius_buffer) && (sq(Xdif)) + sq((Ydif)) > sq(r-radius_buffer)) {
    return true;
  } else {
    return false;
  }
}

void evenCircleLayout() {
  // radii evenly spaced
  for (int i=0; i<lim; i++) {
    stroke(80);
    strokeWeight(0.5);
    noFill();
    arc(width/2, height/2-vertical_offset, map(i, 0, lim, max_rad, 0), map(i, 0, lim, max_rad, 0), -4*PI/3, PI/3, PIE);
  }
}

void sqrtCircleLayout() {
  // radii proportional to sqrt of checkout counts
  for (int i=0; i<lim; i++) {
    stroke(80);
    strokeWeight(0.5);
    noFill();
    arc(width/2, height/2-vertical_offset, map(sqrt(c1.get(i).checkout_counts), 0, sqrt(max_counts), 0, max_rad), map(sqrt(c1.get(i).checkout_counts), 0, sqrt(max_counts), 0, max_rad), -4*PI/3, PI/3, PIE);
  }
}

void rolloverPlot(boolean toggle) {
  for (int i=0; i<lim; i++) { 
    Xdif = mouseX - width/2;
    Ydif = mouseY - (height/2 - vertical_offset);
    if (toggle) {
      r = map(i, 0, lim, max_rad, 0)/2;
    } else {
      r = map(sqrt(c1.get(i).checkout_counts), 0, sqrt(max_counts), 0, max_rad)/2;
      // putting in this offset to prevent circles from being too close together
      if (sqrt(c1.get(i).checkout_counts) - sqrt(c1.get(i+1).checkout_counts) <= 2) {
        r = r+3;
      }
    }
    if (rollover(mouseX, mouseY, toggle)) {

      // print country name in top right corner
      fill(colors[i % (colors.length-1)]);
      textFont(f1);
      textAlign(RIGHT, TOP);
      text(c1.get(i).country_name, width-15, 10);

      // print max counts in lower right corner
      fill(230);
      textFont(f3);
      textAlign(RIGHT, BOTTOM);
      text((int)count_sums[c1.get(i).id], width-15, height-10);


      // determine min and max checkouts in a month
      min = 1000000;
      max = 0;
      for (int j=0; j<data_matrix[0].length; j++) {
        if (data_matrix[c1.get(i).id][j] < min) {
          min = data_matrix[c1.get(i).id][j];
        }
        if (data_matrix[c1.get(i).id][j] > max) {
          max = data_matrix[c1.get(i).id][j];
        }
      }

      // plot rollover graphs
      // shrink the size of the graphs if they go off the screen
      fill(colors[i % (colors.length-1)], 0.2*255);
      float graph_shrink_factor;
      if ( max <= height/2) {
        graph_shrink_factor = 1;
      } else {
        graph_shrink_factor = 0.65;
      }

      beginShape();
      pushMatrix();
      stroke(120);
      translate(width/2, height/2-vertical_offset);
      curveVertex(0, 0);
      curveVertex(0, 0);
      curveVertex(((r-min)+graph_shrink_factor*data_matrix[c1.get(i).id][0])*cos(map(0, 0, data_matrix[0].length-1, -4*PI/3, PI/3)), ((r-min)+graph_shrink_factor*data_matrix[c1.get(i).id][0])*sin(map(0, 0, data_matrix[0].length-1, -4*PI/3, PI/3)));
      for (int k=0; k<data_matrix[0].length-1; k++) {
        curveVertex(((r-min)+graph_shrink_factor*data_matrix[c1.get(i).id][k])*cos(map(k, 0, data_matrix[0].length-1, -4*PI/3, PI/3)), ((r-min)+graph_shrink_factor*data_matrix[c1.get(i).id][k])*sin(map(k, 0, data_matrix[0].length-1, -4*PI/3, PI/3)));
      }
      curveVertex(((r-min)+graph_shrink_factor*data_matrix[c1.get(i).id][data_matrix[0].length-1])*cos(map(data_matrix[0].length-1, 0, data_matrix[0].length-1, -4*PI/3, PI/3)), ((r-min)+graph_shrink_factor*data_matrix[c1.get(i).id][data_matrix[0].length-1])*sin(map(data_matrix[0].length-1, 0, data_matrix[0].length-1, -4*PI/3, PI/3)));
      curveVertex(0, 0);
      curveVertex(0, 0);
      endShape();
      popMatrix();

      // show top 8 items for each country
      textFont(f2);
      textAlign(LEFT, CENTER);
      for (int m=0; m<c1.get(i).items.size(); m++) {
        strokeWeight(10);
        switch (c1.get(i).items.get(m).item_type) {
        case "bk":
          stroke(#F48380, 62);
          line(10, height-(c1.get(i).items.size()-m+1.5)*20+3, 10+map(c1.get(i).items.get(m).bibcounts, 0, max_item_count, 0, 40), height-(c1.get(i).items.size()-m+1.5)*20+3);
          break;
        case "cd":
          stroke(#47B995, 62);
          line(10, height-(c1.get(i).items.size()-m+1.5)*20+3, 10+map(c1.get(i).items.get(m).bibcounts, 0, max_item_count, 0, 40), height-(c1.get(i).items.size()-m+1.5)*20+3);
          break;
        case "dvd":
          stroke(#9FAF4F, 62);
          line(10, height-(c1.get(i).items.size()-m+1.5)*20+3, 10+map(c1.get(i).items.get(m).bibcounts, 0, max_item_count, 0, 40), height-(c1.get(i).items.size()-m+1.5)*20+3);
          break;
        case "vhs":
          stroke(#6DADD0, 62);
          line(10, height-(c1.get(i).items.size()-m+1.5)*20+3, 10+map(c1.get(i).items.get(m).bibcounts, 0, max_item_count, 0, 40), height-(c1.get(i).items.size()-m+1.5)*20+3);
          break;
        case "":
          break;
        }
        fill(165);
        text(c1.get(i).items.get(m).title, 65, height-(c1.get(i).items.size()-m+1.5)*20);
      }
      textFont(f3, 12);
      strokeWeight(10);
      stroke(#F48380, 62);
      line(10+textWidth("book")+10, height-15, 10+textWidth("book")+12, height-15);
      fill(165);
      text("book", 10, height-18);

      strokeWeight(10);
      stroke(#47B995, 62);
      line(10+textWidth("book")+12+10+textWidth("cd")+10, height-15, 10+textWidth("book")+12+10+textWidth("cd")+12, height-15);
      fill(165);
      text("cd", 10+textWidth("book")+12+10, height-18);

      strokeWeight(10);
      stroke(#9FAF4F, 62);
      line(10+textWidth("book")+12+10+textWidth("cd")+12+10+textWidth("dvd")+10, height-15, 10+textWidth("book")+12+10+textWidth("cd")+12+10+textWidth("dvd")+12, height-15);
      fill(165);
      text("dvd", 10+textWidth("book")+12+10+textWidth("cd")+12+10, height-18);

      strokeWeight(10);
      stroke(#6DADD0, 62);
      line(10+textWidth("book")+12+10+textWidth("cd")+12+10+textWidth("dvd")+12+10+textWidth("vhs")+10, height-15, 10+textWidth("book")+12+10+textWidth("cd")+12+10+textWidth("dvd")+12+10+textWidth("vhs")+12, height-15);
      fill(165);
      text("vhs", 10+textWidth("book")+12+10+textWidth("cd")+12+10+textWidth("dvd")+12+10, height-18);
    }
  }
}


void keyPressed() {
  if (keyPressed) {
    toggle = !toggle;
  }
}
